/* $Id: hashtable_test.c,v 1.18 2000/04/06 07:26:53 jm Exp $
 * Hashtable module tests.
 *
 * Dynamic hierarchial IP tunnel
 * Copyright (C) 1998-2000, Dynamics group
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. See README and COPYING for
 * more details.
 */

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h> /* required for netinet/in.h in AMIGAOS */
#include <netinet/in.h>
#include <string.h>
#include <assert.h>

#include "hashtable.h"
#include "list.h"

/* defines */

#define ASSERT assert

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

/* NOTE:
 * The test cases have been numbered according to the module test codes.
 * hashtable module uses list module, so even the module testing for hashtable
 * is sort of integration testing.
 */

/* structures */
/* NOTE: YOU ONLY CAN ADD THIS STRUCTURE TO ONE LIST AT A TIME! */
/* SO DO NOT TRY TO ADD THIS KIND OF A STRUCTURE INTO TWO HASHTABLES! */

struct testdata {
	struct node node;
	int keyval;
	char *text;
	float x;
};

struct hashtable * test_hashtable_init(int size)
{
	struct hashtable *table = hashtable_init(size);
	if (table == NULL) {
		printf("Test hashtable_init: Init failed, table was NULL\n");
		return NULL;
	}
	printf("test_hashtable_init: Init successful!\n");
	return table;
}

/* A hashfunc to be given as an argument. */
/* Returns "key mod size". */

int test_hash(void *key, const int size)
{
	int i;
	struct testdata *data;

	data = (struct testdata *) key;
	i = data->keyval;
	printf("Key mod size = %d mod %d = %d \n", i, size, (i % size));
	ASSERT(i % size >= 0);
	ASSERT(i % size < size);
	return (i % size);
}

/* A comparison for the testdata. */
/* Returns 1 if the fields of the structs key and cmpd are the same.
 * Otherwise will return 0. */

int test_cmp(void *key, struct node *cmpd)
{
	struct testdata *data1, *data2;

	if (key == NULL || cmpd == NULL) {
		printf("\ntest_cmp: one of the given arguments was NULL !\n"
		       "No match! Returning 0 !\n");
		return 0;
	}

	data1 = (struct testdata *) key;
	data2 = (struct testdata *) cmpd;
	if (data1->keyval == data2->keyval &&
	    data1->text == data2->text &&
	    data1->x == data2->x) {
                printf("test_cmp: the given key matches to the compared "
		       "struct\n");
		return 1;
	}
	printf("test_cmp: no match (yet)...\n");
	return 0;
}

/* An invalid hash function */

int hashfunc_invalid(void *key, const int size)
{
	return -5;
}

/* A valid hash function */

int hashfunc_valid(void *key, const int size)
{
	struct testdata *data;

	data = (struct testdata *) key;
	return data->keyval % size;
}

/* A valid compare function */

int cmpfunc_valid(void *key, struct node *cmpd)
{
	struct testdata *data1, *data2;

	data1 = (struct testdata *) key;
	data2 = (struct testdata *) cmpd;
	printf("cmpfunc_valid: data1 = 0x%08x, data2 = 0x%08x\n",
	       (int) data1, (int) data2);
	return data1->keyval == data2->keyval;
}

static int iterfunc_return = TRUE;

/* An iterator function that prints some information */

int iterfunc_print(struct node *node, void *dummy)
{
	struct testdata *data;

	data = (struct testdata *) node;
	printf("iterfunc_print: keyval = %d, text = '%s', x = %f\n",
	       data->keyval, data->text, data->x);
	return iterfunc_return;
}

/* An iterator function that removes the node */

int iterfunc_remove(struct node *node, void *dummy)
{
	struct testdata *data;

	data = (struct testdata *) node;
	printf("iterfunc_remove: keyval = %d, text = '%s', x = %f\n",
	       data->keyval, data->text, data->x);
	hashtable_remove(&data->node);
	return iterfunc_return;
}

/* An iterator function that verifies the data */

int iterfunc_verify(struct node *node, void *data)
{
	if (data == (void *) iterfunc_verify) {
		return 1;
	}
	return 0;
}

/******************************************
 *                                        *
 *     Hashtable test main function       *
 *                                        *
 ******************************************/

int main(void)
{
	struct testdata *data1, *data2, *data3, *data4, *data5;
	int i, n, size, primesize;
	int result1, result2, result3, result4, result5;
	struct hashtable *A, *B;
	struct testdata *data1_out, *data2_out;

	struct hashtable *table;
	struct testdata *data;

	printf("\n\t **********************************\n"
	       "\t  STARTING HASHTABLE MODULE TESTS! \n"
	       "\n"
	       "\t ***********************************\n");

	/* ASSIGNING TEST DATA VALUES: */

	printf("\n\t ASSIGNING TEST DATA VALUES:\n\n");

	data1 = malloc(sizeof(struct testdata));
	ASSERT(data1 != NULL);


	list_init_node(&data1->node);
	data1->keyval = 5;
	data1->text = "Matti Meikalaenen";
	data1->x = 123.4444;
	printf("Testdata1: %d, %s, %f\n\n",
	       data1->keyval, data1->text, data1->x);


	data2 = malloc(sizeof(struct testdata));
	ASSERT(data2 != NULL);
	list_init_node(&data2->node);
	data2->keyval = 111257;
	data2->text = "Jooseppi Ikaeheimo";
	data2->x = 1999.25;
	printf("Testdata2: %d, %s, %f\n\n",
	       data2->keyval, data2->text, data2->x);


	data3 = malloc(sizeof(struct testdata));
	ASSERT(data3 != NULL);
	list_init_node(&data3->node);
	data3->keyval = 111257;
	data3->text = "Hemmo Heikinheimo";
	data3->x = 0.1212122;
	printf("Testdata3: %d, %s, %f\n\n",
	       data3->keyval, data3->text, data3->x);


	data4 = malloc(sizeof(struct testdata));
	ASSERT(data4 != NULL);
	list_init_node(&data4->node);
	data4->keyval = 123450;
	data4->text = "Kimmon data4";
	data4->x = 3.14159265;
	printf("Testdata4: %d, %s, %f\n\n",
	       data4->keyval, data4->text, data4->x);


	data5 = malloc(sizeof(struct testdata));
	ASSERT(data5 != NULL);
	list_init_node(&data5->node);
	data5->keyval = 123456;
	data5->text = "Kimmon data5";
	data5->x = 3.14159266;
	printf("Testdata5: %d, %s, %f\n\n",
	       data5->keyval, data5->text, data5->x);

	printf("Testing hashtable-module...\n");

	/* M01 */
	printf("Test case M01: Initialize, size is zero\n");
	ASSERT(hashtable_init(0) == NULL);
	printf("M01 test cases (1) OK\n");

	/* M02 */
	printf("Test case M02: Initialize, size is negative\n");
	ASSERT(hashtable_init(-5) == NULL);
	printf("M02 test cases (1) OK\n");

	/* M03 */
	printf("Test case M03 requires manual testing\n");

	/* M04 */
	printf("Test case M04: Initialize, size is too large\n");
	ASSERT(hashtable_init(20000000) == NULL);
	printf("M04 test cases (1) OK\n");

	/* M05 */
	printf("Test case M05: Initialize, realistic values\n");
	table = hashtable_init(200);
	ASSERT(table != NULL);
	printf("M05 test cases (1) OK\n");

	/* M06 */
	printf("Test case M06: Destroy hashtable when it's not empty\n");
	ASSERT(hashtable_add(table, hashfunc_valid, data1,
			     &data1->node) == TRUE);
	ASSERT(hashtable_destroy(table) == FALSE);
	printf("M06 test cases (1) OK\n");

	table = hashtable_init(200);
	ASSERT(table != NULL);

	/* M07 */
	printf("Test case M07: Destroy hashtable, table is empty\n");
	ASSERT(hashtable_destroy(table) == TRUE);
	printf("M07 test cases (1) OK\n");

	table = hashtable_init(200);
	ASSERT(table != NULL);

	/* M08 */
	printf("Test case M08: Add, hash function invalid\n");
	ASSERT(hashtable_add(table, hashfunc_invalid, data1,
			     &data1->node) == FALSE);
	printf("M08 test cases (1) OK\n");

	/* M09 */
	printf("Test case M09: Add, hash function valid\n");
	ASSERT(hashtable_add(table, hashfunc_valid, data1,
			     &data1->node) == TRUE);
	printf("M09 test cases (1) OK\n");

	/* M10 */
	printf("Test case M10: Fetch, hash function invalid\n");
	data1->keyval = 5;
	data = (struct testdata *) hashtable_fetch(table, hashfunc_invalid,
						   data1, cmpfunc_valid);
	ASSERT(data == NULL);
	printf("M10 test cases (1) OK\n");

	/* M11 */
	printf("Test case M11: Fetch, hash function valid, node found\n");
	data = (struct testdata *) hashtable_fetch(table, hashfunc_valid,
						   data1, cmpfunc_valid);
	ASSERT(&data->node == &data1->node);
	printf("M11 test cases (1) OK\n");

	/* M12 */
	printf("Test case M12: Fetch, hash function valid, node not found\n");
	data = (struct testdata *) hashtable_fetch(table, hashfunc_valid,
						   data2, cmpfunc_valid);
	ASSERT(data == NULL);
	printf("M12 test cases (1) OK\n");

	/* M13 */
	printf("Test case M13: Remove from table\n");
	hashtable_remove(&data1->node);
	printf("M13 test cases (1) OK\n");

	/* M14 */
	printf("Test case M14: Find and remove from table, node found\n");
	data1->keyval = 5;
	ASSERT(hashtable_add(table, hashfunc_valid, data1,
			     &data1->node) == TRUE);
	ASSERT(hashtable_find_and_remove(table, hashfunc_valid, data1,
					 cmpfunc_valid) == TRUE);
	printf("M14 test cases (1) OK\n");

	/* M15 */
	printf("Test case M15: Find and remove from table, node not found\n");
	data1->keyval = 5;
	ASSERT(hashtable_add(table, hashfunc_valid, data1,
			     &data1->node) == TRUE);
	data1->keyval = 6;
	ASSERT(hashtable_find_and_remove(table, hashfunc_valid, data1,
					 cmpfunc_valid) == FALSE);
	printf("M15 test cases (1) OK\n");

	data1->keyval = 5;
	ASSERT(hashtable_find_and_remove(table, hashfunc_valid, data1,
					 cmpfunc_valid) == TRUE);

	/* M16 */
	printf("Test case M16: Use iterator on an empty hashtable\n");
	ASSERT(hashtable_iterator(table, iterfunc_print, NULL) == TRUE);
	printf("M16 test cases (1) OK\n");

	/* M17 */
	printf("Test case M17: Use iterator on a non-empty hashtable\n");
	ASSERT(hashtable_add(table, hashfunc_valid, data1,
			     &data1->node) == TRUE);
	ASSERT(hashtable_add(table, hashfunc_valid, data2,
			     &data2->node) == TRUE);
	ASSERT(hashtable_add(table, hashfunc_valid, data3,
			     &data3->node) == TRUE);
	ASSERT(hashtable_add(table, hashfunc_valid, data4,
			     &data4->node) == TRUE);
	ASSERT(hashtable_add(table, hashfunc_valid, data5,
			     &data5->node) == TRUE);
	ASSERT(hashtable_iterator(table, iterfunc_print, NULL) == TRUE);
	printf("M17 test cases (1) OK\n");

	/* M18 */
	printf("Test case M18: Use iterator to remove entries\n");
	ASSERT(hashtable_iterator(table, iterfunc_remove, NULL) == TRUE);
	printf("M18 test cases (1) OK\n");

	/* same tests (M16 .. M18) with iterfunc that returns FALSE */
	iterfunc_return = FALSE;

	/* M19 */
	printf("Test case M19: Use iterator on an empty hashtable "
	       "(iterfunc returns FALSE)\n");
	/* iterfunc should not be called, so TRUE is returned */
	ASSERT(hashtable_iterator(table, iterfunc_print, NULL) == TRUE);
	printf("M19 test cases (1) OK\n");

	/* M20 */
	printf("Test case M20: Use iterator on a non-empty hashtable "
	       "(iterfunc returns FALSE)\n");
	ASSERT(hashtable_add(table, hashfunc_valid, data1,
			     &data1->node) == TRUE);
	ASSERT(hashtable_add(table, hashfunc_valid, data2,
			     &data2->node) == TRUE);
	ASSERT(hashtable_add(table, hashfunc_valid, data3,
			     &data3->node) == TRUE);
	ASSERT(hashtable_add(table, hashfunc_valid, data4,
			     &data4->node) == TRUE);
	ASSERT(hashtable_add(table, hashfunc_valid, data5,
			     &data5->node) == TRUE);
	ASSERT(hashtable_iterator(table, iterfunc_print, NULL) == FALSE);
	printf("M20 test cases (1) OK\n");

	/* M21 */
	printf("Test case M21: Use iterator to remove entries "
	       "(iterfunc returns FALSE)\n");
	ASSERT(hashtable_iterator(table, iterfunc_remove, NULL) == FALSE);
	printf("M21 test cases (1) OK\n");

	iterfunc_return = TRUE;
	ASSERT(hashtable_iterator(table, iterfunc_remove, NULL) == TRUE);

	/* M22 */
	printf("Test case M22: Find prime with negative starting value\n");
	ASSERT(hashtable_find_prime(-5) == 0);
	printf("M22 test cases (1) OK\n");

	/* M23 */
	printf("Test case M23: Find prime with zero starting value\n");
	ASSERT(hashtable_find_prime(0) == 0);
	printf("M23 test cases (1) OK\n");

	/* M24 */
	printf("Test case M24: Find prime with 1, 2 and 3 starting value\n");
	ASSERT(hashtable_find_prime(1) == 2);
	ASSERT(hashtable_find_prime(2) == 2);
	ASSERT(hashtable_find_prime(3) == 3);
	printf("M24 test cases (3) OK\n");

	/* M25 */
	printf("Test case M25: Find prime with already a prime as the "
	       "starting value\n");
	ASSERT(hashtable_find_prime(113) == 113);
	printf("M25 test cases (1) OK\n");

	/* M26 */
	printf("Test case M26: Find prime with a non-prime as the starting "
	       "value\n");
	ASSERT(hashtable_find_prime(4) == 5);
	printf("M26 test cases (1) OK\n");

	/* M27 */
	printf("Test case M27: Find prime with a large starting value\n");
	ASSERT(hashtable_find_prime((1 << 29) + 10) == 0);
	printf("M27 test cases (1) OK\n");

	/* M28 */
	printf("Test case M28: Verify that data is passed to iterator "
	       "function\n");
	ASSERT(hashtable_add(table, hashfunc_valid, data1,
			     &data1->node) == TRUE);
	ASSERT(hashtable_iterator(table, iterfunc_verify,
				  (void *) iterfunc_verify) == TRUE);
	hashtable_remove(&data1->node);
	printf("M28 test cases (1) OK\n");


	/* XM01 */
	/* print 100 primes */

	n = 1;
	for (i = 0; i < 100; i++) {
		n = hashtable_find_prime(n);
		printf("%d\t", n);
		n++;
	}
	printf("\n");

	/*  SETTING HASHTABLE SIZE: */

	printf("\n\t	SETTING HASHTABLE SIZE:\n\n ");

	size = 200;

	/* XM02 */
	/* Test the hashtable_find_prime
	 * function with non prime argument */

	primesize = hashtable_find_prime(size);
	printf("With size %d the hashtable size will be set to next "
	       "prime %d\n", size, primesize);

	ASSERT(primesize >= size);


	/* INITIALIZING HASTABLES:  */

	printf("\n\t INITIALIZING HASTABLES:\n\n");

	/* XM03 */
	/* Test hastable_init function */
	A = hashtable_init(primesize);
	B = hashtable_init(size);


	/* ADDING DATA: */

	printf("\n\t ADDING DATA TO HASHTABLES:\n\n"
	       " \t Result of 1 = Success,\n"
	       " \t Result of 0 = Failure\n\n");

	/* XM04 */
	/* Add different data to two different hashtables. */

        result1 = hashtable_add(A, test_hash, data1, &data1->node);
	printf("Adding data1 to hashtable A :\n"
	       "data1->keyval = %d,\n"
	       "data1->text = %s,\n"
	       "\nThe result of adding data1 is:\t %d.\n\n",
	       data1->keyval, data1->text, result1);

	result2 = hashtable_add(B, test_hash, data2, &data2->node);
	printf("Adding data2 to hashtable B :\n"
	       "data2->keyval = %d,\n"
	       "data2->text = %s,\n"
	       "\nThe result of adding data2 is:\t %d.\n\n",
	       data2->keyval, data2->text, result2);


	result3 = hashtable_add(B, test_hash, data3, &data3->node);
	printf("Adding data3 to hashtable B :\n"
	       "data3->keyval = %d,\n"
	       "data3->text = %s,\n"
	       "\nThe result of adding data3 is:\t %d.\n\n",
	       data3->keyval, data3->text, result3);


	result4 = hashtable_add(A, test_hash, data4, &data4->node);
	printf("Adding data4 to hashtable A :\n"
	       "data4->keyval = %d,\n"
	       "data4->text = %s,\n"
	       "\nThe result of adding data4 is:\t %d.\n\n",
	       data4->keyval, data4->text, result4);


	result5 = hashtable_add(A, test_hash, data5, &data5->node);
	printf("Adding data5 to hashtable A :\n"
	       "data5->keyval = %d,\n"
	       "data5->text = %s,\n"
	       "\nThe result of adding data5 is:\t %d.\n\n",
	       data5->keyval, data5->text, result5);



	/* FETCHING OF DATA FROM THE HASHTABLE: */


	printf("\n\t FETCHING OF DATA FROM THE HASHTABLE:\n\n");


	/* XM05 */
	/* Trying to fetch data2 out of the hashtable. */

	printf("Trying to fetch data2 out of the hashtable B!\n\n");

	data2_out = (struct testdata *) hashtable_fetch(B, test_hash, data2,
							test_cmp);
	printf("\nLet's test the fetch result one more time:\n\n");
	if (data2_out == NULL) {
		printf("data2_out == NULL !!!\n");
                ASSERT(FALSE);
	}
	printf("data2_out->keyval = %d\n", data2_out->keyval);
	printf("data2_out->text = %s\n", data2_out->text);
	printf("data2_out->x = %f\n", data2_out->x);

	printf("\nComparing the fetched and original\n"
	       " data with the comparefunction.\n");
	test_cmp(data2_out, &data2->node);

	/* XM06 */
	/* FETCHING SOMETHING INEXISTENT: */

	printf("\nTrying to fetch data1 out of the hashtable B,"
	       "which does NOT contain data1 !\n\n");
	data1_out = (struct testdata *) hashtable_fetch(B, test_hash,
							data1, test_cmp);

	if (data1_out == NULL) {
		printf("\ndata1_out == NULL\n\n");
                /* ASSERT(FALSE); */
	}
	printf("\nComparing the fetched (SHOULD BE NULL!)\n"
	       "and original data with the comparefunction.\n");
	test_cmp(data1_out, &data1->node);

	/* REMOVING DATA: */

	/* XM07 */
	/* Fetch and remove the data with one hashtable - function call. */

	printf("\n\t REMOVING DATA:\n\n");

	printf("Trying to find and remove data1 from hashtable A.\n\n");
	result1 = hashtable_find_and_remove(A, test_hash, data1, test_cmp);
	printf("Result of finding and deleting data 1 from hashtable A:\n "
	       "Result = %d.\n", result1);


	/* XM08 */
	/* Remove data */

	hashtable_remove(&data2->node);
	hashtable_remove(&data3->node);
	hashtable_remove(&data4->node);
	hashtable_remove(&data5->node);

	/* XM09 */
	/* Adding the previously removed data back. */

	/* ADDING DATA AGAIN: */

	printf("\n\t ADDING DATA AGAIN:\n\n");

        result1 = hashtable_add(A, test_hash, data1, &data1->node);
	printf("Result of adding data 1 to hashtable A is %d.\n",result1);

        result2 = hashtable_add(A, test_hash, data2, &data2->node);
	printf("Result of adding data 2 to hashtable A is %d.\n",result2);
        result3 = hashtable_add(A, test_hash, data3, &data3->node);
	printf("Result of adding data 3 to hashtable A is %d.\n",result3);
        result4 = hashtable_add(A, test_hash, data4, &data4->node);
	printf("Result of adding data 4 to hashtable A is %d.\n",result4);
        result5 = hashtable_add(A, test_hash, data5, &data5->node);
	printf("Result of adding data 5 to hashtable A is %d.\n",result5);



	/* XM10 */
	/* Fetching and removing with consecutive function calls. */

	/* THEN FETCHING AGAIN AND REMOVING AFTER THAT: */

	printf("\n\t THEN FETCHING AGAIN AND REMOVING AFTER THAT:\n\n");

	printf("Trying to fetch data3 out of the hashtable A!\n\n");
	data1_out = (struct testdata *)
		hashtable_fetch(A, test_hash, data3, test_cmp);
	ASSERT(data1_out == data3);
	printf("OK!\n");
	hashtable_remove(&data1_out->node);


	printf("\n Trying to fetch data1 out of the hashtable A!\n\n");
	data1_out = (struct testdata *)
		hashtable_fetch(A, test_hash, data1, test_cmp);
	ASSERT(data1_out == data1);
	printf("OK!\n");
	hashtable_remove(&data1_out->node);


	printf("\n Trying to fetch data4 out of the hashtable A!\n\n");
	data1_out = (struct testdata *)
		hashtable_fetch(A, test_hash, data4, test_cmp);
	ASSERT(data1_out == data4);
	printf("OK!\n");
	hashtable_remove(&data1_out->node);

	printf("\n Trying to fetch data5 out of the hashtable A!\n\n");
	data1_out = (struct testdata *)
		hashtable_fetch(A, test_hash, data5, test_cmp);
	ASSERT(data1_out == data5);
	printf("OK!\n");
	hashtable_remove(&data1_out->node);

	printf("\n Trying to fetch data2 out of the hashtable A!\n\n");
	data1_out = (struct testdata *)
		hashtable_fetch(A, test_hash, data2, test_cmp);
	ASSERT(data1_out == data2);
	printf("OK!\n");
	hashtable_remove(&data1_out->node);


	/* XM11 */
	/* Trying to destroy a hashtable. */

	printf("\n Destroying hashtables A and B:\n");
	ASSERT(hashtable_destroy(B) == TRUE);
	ASSERT(hashtable_destroy(A) == TRUE);

	printf("\nFreeing memory:\n");

	free(data1);
	free(data2);
	free(data3);
	free(data4);
	free(data5);

	printf("\n\t*****************************************\n"
	       "\t*                                       *\n"
	       "\t*    HASHTABLE TEST FINISHED            *\n"
	       "\t*                                       *\n"
	       "\t*****************************************\n\n");
	return 0;
}
